@@ -1,9 +1,9 @@
 #
-# $Id: Encode.pm,v 2.6 2004/10/22 06:23:11 dankogai Exp dankogai $
+# $Id: Encode.pm,v 2.7 2004/10/22 19:35:52 dankogai Exp $
 #
 package Encode;
 use strict;
-our $VERSION = do { my @r = (q$Revision: 2.6 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+our $VERSION = do { my @r = (q$Revision: 2.7 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
 sub DEBUG () { 0 }
 use XSLoader ();
 XSLoader::load(__PACKAGE__, $VERSION);
@@ -554,43 +554,51 @@ method.
   perlio_ok("euc-jp")
 
 Fortunately, all encodings that come with Encode core are PerlIO-savvy
-except for hz and ISO-2022-kr.  For gory details, see L<Encode::Encoding> and L<Encode::PerlIO>.
+except for hz and ISO-2022-kr.  For gory details, see
+L<Encode::Encoding> and L<Encode::PerlIO>.
 
 =head1 Handling Malformed Data
 
 The optional I<CHECK> argument is used as follows.  When you omit it,
-Encode::FB_DEFAULT is assumed.
+Encode::FB_DEFAULT ( == 0 ) is assumed.
 
-B<Note:> Not all encoding suppport this feature.  Some encodings
-ignore I<CHECK> argument.  For example, L<Encode::Unicode> ignores
-I<CHECK> and it always croaks on error.
+=over 2
+
+=item B<NOTE:> Not all encoding suppport this feature
+
+Some encodings ignore I<CHECK> argument.  For example,
+L<Encode::Unicode> ignores I<CHECK> and it always croaks on error.
+
+=back
+
+Now here is the list of I<CHECK> values available
 
 =over 2
 
 =item I<CHECK> = Encode::FB_DEFAULT ( == 0)
 
-If I<CHECK> is 0, (en|de)code will put a I<substitution character>
-in place of a malformed character.  For UCM-based encodings,
-E<lt>subcharE<gt> will be used.  For Unicode, the code point C<0xFFFD> is used.
-If the data is supposed to be UTF-8, an optional lexical warning
-(category utf8) is given.
+If I<CHECK> is 0, (en|de)code will put a I<substitution character> in
+place of a malformed character.  When you encode to UCM-based encodings,
+E<lt>subcharE<gt> will be used.  When you decode from UCM-based
+encodings, the code point C<0xFFFD> is used.  If the data is supposed
+to be UTF-8, an optional lexical warning (category utf8) is given.
 
 =item I<CHECK> = Encode::FB_CROAK ( == 1)
 
 If I<CHECK> is 1, methods will die on error immediately with an error
 message.  Therefore, when I<CHECK> is set to 1,  you should trap the
-fatal error with eval{} unless you really want to let it die on error.
+error with eval{} unless you really want to let it die.
 
 =item I<CHECK> = Encode::FB_QUIET
 
 If I<CHECK> is set to Encode::FB_QUIET, (en|de)code will immediately
-return the portion of the data that has been processed so far when
-an error occurs. The data argument will be overwritten with
-everything after that point (that is, the unprocessed part of data).
-This is handy when you have to call decode repeatedly in the case
-where your source data may contain partial multi-byte character
-sequences, for example because you are reading with a fixed-width
-buffer. Here is some sample code that does exactly this:
+return the portion of the data that has been processed so far when an
+error occurs. The data argument will be overwritten with everything
+after that point (that is, the unprocessed part of data).  This is
+handy when you have to call decode repeatedly in the case where your
+source data may contain partial multi-byte character sequences,
+(i.e. you are reading with a fixed-width buffer). Here is a sample
+code that does exactly this:
 
   my $data = ''; my $utf8 = '';
   while(defined(read $fh, $buffer, 256)){
@@ -621,8 +629,8 @@ where I<HHHH> is the Unicode ID of the character that cannot be found
 in the character repertoire of the encoding.
 
 HTML/XML character reference modes are about the same, in place of
-C<\x{I<HHHH>}>, HTML uses C<&#I<NNNN>>; where I<NNNN> is a decimal digit and
-XML uses C<&#xI<HHHH>>; where I<HHHH> is the hexadecimal digit.
+C<\x{I<HHHH>}>, HTML uses C<&#I<NNNN>;> where I<NNNN> is a decimal digit and
+XML uses C<&#xI<HHHH>;> where I<HHHH> is the hexadecimal digit.
 
 =item The bitmask
 
