@@ -1,7 +1,7 @@
 package Encode::Encoding;
 # Base class for classes which implement encodings
 use strict;
-our $VERSION = do { my @r = (q$Revision: 1.32 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+our $VERSION = do { my @r = (q$Revision: 2.1 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
 
 require Encode;
 
@@ -14,8 +14,10 @@ sub Define
     Encode::define_encoding($obj, $canonical, @_);
 }
 
-sub name         { return shift->{'Name'} }
-sub new_sequence { return $_[0] }
+sub name  { return shift->{'Name'} }
+
+sub renew { return $_[0] }
+*new_sequence = \&renew;
 
 sub needs_lines { 0 };
 
@@ -24,7 +26,8 @@ sub perlio_ok {
     return $@ ? 0 : 1;
 }
 
-# Temporary legacy methods
+# (Temporary|legacy) methods
+
 sub toUnicode    { shift->decode(@_) }
 sub fromUnicode  { shift->encode(@_) }
 
@@ -36,14 +39,14 @@ sub encode {
     require Carp;
     my $obj = shift;
     my $class = ref($obj) ? ref($obj) : $obj;
-    Carp::croak $class, "->encode() not defined!";
+    Carp::croak($class . "->encode() not defined!");
 }
 
 sub decode{
     require Carp;
     my $obj = shift;
     my $class = ref($obj) ? ref($obj) : $obj;
-    Carp::croak $class, "->encode() not defined!";
+    Carp::croak($class . "->encode() not defined!");
 }
 
 sub DESTROY {}
@@ -160,15 +163,28 @@ Predefined As:
 
 MUST return the string representing the canonical name of the encoding.
 
-=item -E<gt>new_sequence
+=item -E<gt>renew
 
 Predefined As:
 
-  sub new_sequence { return $_[0] }
+  sub renew { return $_[0] }
 
-This is a placeholder for encodings with state. It should return an
-object which implements this interface.  All current implementations
-return the original object.
+This method reconstructs the encoding object if necessary.  If you need
+to store the state during encoding, this is where you clone your object.
+Here is an example:
+
+  sub renew { 
+      my $self = shift;
+      my $clone = bless { %$self } => ref($self);
+      $clone->{clone} = 1; # so the caller can see it
+      return $clone;
+  }
+
+Since most encodings are stateless the default behavior is just return
+itself as shown above.
+
+PerlIO ALWAYS calls this method to make sure it has its own private
+encoding object.
 
 =item -E<gt>perlio_ok()
 
